"""Adding english description for Phase, Group and Team

Revision ID: 314114c0b886
Revises: 045a9142794e
Create Date: 2023-05-26 23:46:38.127268

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "314114c0b886"
down_revision = "045a9142794e"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("group", sa.Column("description_en", sa.String(length=100), nullable=False))
    op.create_unique_constraint(None, "group", ["description_en"])
    op.add_column("phase", sa.Column("description_en", sa.String(length=100), nullable=False))
    op.add_column("team", sa.Column("description_en", sa.String(length=100), nullable=False))
    op.create_unique_constraint(None, "team", ["description_en"])
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, "team", type_="unique")
    op.drop_column("team", "description_en")
    op.drop_column("phase", "description_en")
    op.drop_constraint(None, "group", type_="unique")
    op.drop_column("group", "description_en")
    # ### end Alembic commands ###
